#ifndef __OBJECT_H__
#define __OBJECT_H__

/**
 * Abstract base object.
 */
class Object
{
  public:
    virtual ~Object (void) {}
};

/**
 * A number.
 */
class Number : public Object
{
  public:
    Number (float value)
      : m_value (value)
    {
    }

  public:
    inline float value (void) const { return m_value; }

  private:
    float m_value;
};

/**
 * An operator.
 */
class Operator : public Object
{
  public:
    virtual float eval (float, float) const = 0;
};

/**
 * Addition.
 */
class Add : public Operator
{
  public:
    virtual float eval (float a, float b) const { return a+b; }
};

/**
 * Substraction.
 */
class Sub : public Operator
{
  public:
    virtual float eval (float a, float b) const { return a-b; }
};

/**
 * Multiplication.
 */
class Mul : public Operator
{
  public:
    virtual float eval (float a, float b) const { return a*b; }
};

/**
 * Division.
 */
class Div : public Operator
{
  public:
    virtual float eval (float a, float b) const { return a/b; }
};

#endif
