#include "object.h"
#include "intermediate.h"

#include <list>
#include <string>
#include <iostream>

/**
 * Display contents if given list.
 * @param prefix A prefix string.
 * @param list The list to print.
 */
template<class S> void print (const std::string &prefix, const std::list<S *> &list)
{
  std::cout << prefix << ": ";
  for (typename std::list<S *>::const_iterator i = list.begin(), end = list.end(); i != end; ++i)
    (*i)->print ();
  std::cout << std::endl;
}

/**
 * Purge the contents of the given list.
 * @param list The list to purge.
 */
template<class S> void purge (std::list<S *> &list)
{
  for (typename std::list<S *>::iterator i = list.begin(), end = list.end(); i != end; ++i)
    delete *i;
  list.clear ();
}

/**
 * Intermediate main routine.
 */
int main (int, char **)
{
  // init
  std::list<Input *> input;
  std::list<Output *> output;
  input.push_back (new A);
  input.push_back (new B);
  input.push_back (new A);
  input.push_back (new A);

  // do the thing
  try
  {
    print<Input> ("INPUT", input);
    intermediate::PatternMatcher ("", false, true, 0).process (input, output);
    print<Output> ("OUTPUT", output);
  }

  // cannot match
  catch (oors::MatchException<Input> &exception)
  {
    std::cerr << exception.msg() << std::endl;
    exception.item()->print ();
  }

  // something bad has happened
  catch (oors::Exception &exception)
  {
    std::cerr << exception.msg() << std::endl;
  }

  // cleanup
  purge<Input> (input);
  purge<Output> (output);

  // done
  return 0;
}
