#include "debugger.h"

#include <oors_internal.h>

#include <iostream>

void Debugger::currentRule (const oors::Rule<Operation, Operation> *rule)
{
  std::cout << std::endl << "Rule<" << rule->name() << ">: Current." << std::endl;
}

void Debugger::finishRule (const oors::Rule<Operation, Operation> *rule, bool matched)
{
  std::cout << "Rule<" << rule->name() << ">: Matched " << (matched?"successfully":"unsuccessfully") << ", Useable alternatives: " << rule->alternatives().size() << "." << std::endl << std::endl;
}

void Debugger::applyRule (const oors::Rule<Operation, Operation> *rule)
{
  std::cout << "Rule<" << rule->name() << ">: Apply." << std::endl;
}

void Debugger::newAlternative (const oors::Alternative<Operation, Operation> *alternative, const oors::Alternative<Operation, Operation> *parent)
{
  std::cout << "Alternative<" << alternative << ">: Derived from " << parent;
  if (parent)
    std::cout << " with cost " << parent->cost() << " and that belongs to rule "+parent->rule()->name();
  std::cout << ", Remaining items: " << alternative->remaining() << "." << std::endl;
}

void Debugger::currentAlternative (const oors::Alternative<Operation, Operation> *alternative)
{
  std::cout << std::endl << "Alternative<" << alternative << ">: Current, Remaining items: " << alternative->remaining() << "." << std::endl;
}

void Debugger::deleteAlternative (const oors::Alternative<Operation, Operation> *alternative)
{
  std::cout << "Alternative<" << alternative << ">: Deleted." << std::endl;
}

void Debugger::checkItemPattern (const oors::Alternative<Operation, Operation> *alternative, unsigned int index, const Operation *item, bool result)
{
  std::cout << "Alternative<" << alternative << ">: ItemPattern " << index << " side condition is " << (result?"true":"false") << " for " << item << " = ";
  item->print ();
}

void Debugger::matchItemPattern (const oors::Alternative<Operation, Operation> *alternative, unsigned int index, const Operation *item)
{
  std::cout << "Alternative<" << alternative << ">: Pattern[" << index << "] = " << item << " = ";
  item->print ();
}

void Debugger::matchWildcardPattern (const oors::Alternative<Operation, Operation> *alternative, unsigned int index, const Operation *item)
{
  std::cout << "Alternative<" << alternative << ">: Pattern[" << index << "] += " << item << " = ";
  item->print ();
}

void Debugger::finishWildcardPattern (const oors::Alternative<Operation, Operation> *alternative, unsigned int index)
{
  std::cout << "Alternative<" << alternative << ">: Pattern[" << index << "] done" << std::endl;
}

void Debugger::finishBasicBlock (int cost)
{
  std::cout << std::endl << std::endl << "Basic block finished with cost " << cost << std::endl;
}
