#include "operation.h"
#include "debugger.h"
#include "minimum.h"

#include <oors_internal.h>

#include <list>
#include <string>
#include <iostream>

/**
 * Minimum main routine.
 * @param argc Argument count.
 * @param argv Argument list.
 */
int main (int argc, char **argv)
{
  // no expression given
  if (argc < 2)
  {
    std::cout << "Usage: " << argv[0] << " <local|global>" << std::endl
              << "Processes a basic block in either local or global cost minimum mode." << std::endl
              << "This example demonstrates the the local cost minimum does not always lead to an optimal result." << std::endl;
    return 1;
  }

  // init
  std::list<Operation *> cpu, gpu;
  cpu.push_back (new CPUOperation (Operation::OP_CHEAP));
  cpu.push_back (new CPUOperation (Operation::OP_EXPENSIVE));
  cpu.push_back (new CPUOperation (Operation::OP_CHEAP));
  cpu.push_back (new CPUOperation (Operation::OP_EXPENSIVE));

  // display what we got
  std::cout << "INPUT:" << std::endl;
  for (std::list<Operation *>::iterator i = cpu.begin(); i != cpu.end(); ++i)
    (*i)->print ();

  // do the thing
  try
  {
    // process
    Debugger debugger;
    minimum::PatternMatcher matcher;
    matcher.setFirstMatch (false);
    matcher.setLocalMinimum (!strcmp(argv[1], "local"));
    matcher.setDebugger (&debugger);
    matcher.process (cpu, gpu);

    // display result
    std::cout << std::endl << "RESULT:" << std::endl;
    for (std::list<Operation *>::iterator i = gpu.begin(), end = gpu.end(); i != end; ++i)
      (*i)->print ();
  }

  // cannot match
  catch (oors::MatchException<Operation> &exception)
  {
    std::cerr << exception.msg() << std::endl;
    exception.item()->print ();
  }

  // something bad has happened
  catch (oors::Exception &exception)
  {
    std::cerr << exception.msg() << std::endl;
  }

  // cleanup
  for (std::list<Operation *>::iterator i = cpu.begin(), end = cpu.end(); i != end; ++i)
    delete *i;
  for (std::list<Operation *>::iterator i = gpu.begin(), end = gpu.end(); i != end; ++i)
    delete *i;

  // done
  return 0;
}
