#ifndef __MINIMUM_H__
#define __MINIMUM_H__

#include <iostream>

class Operation
{
  public:
    typedef enum
    {
      OP_CHEAP,
      OP_EXPENSIVE
    } Opcode;

  public:
    Operation (Opcode opcode)
      : m_opcode (opcode)
    {
    }

   virtual ~Operation (void)
   {
   }

  public:
    virtual void print (void) const = 0;
    virtual Operation *copy (void) const = 0;

  public:
    inline Opcode opcode (void) const { return m_opcode; }

  protected:
    Opcode m_opcode;
};

class CPUOperation : public Operation
{
  public:
    CPUOperation (Opcode opcode)
      : Operation (opcode)
    {
    }

  public:
    void print (void) const { std::cout << "CPUOperation<" << ((m_opcode == OP_CHEAP)?"OP_CHEAP":"OP_EXPENSIVE") << "> " << this << std::endl; }
    Operation *copy (void) const { return new CPUOperation (m_opcode); }
};

class GPUOperation : public Operation
{
  public:
    GPUOperation (Opcode opcode)
      : Operation (opcode)
    {
    }

  public:
    void print (void) const { std::cout << "GPUOperation<" << ((m_opcode == OP_CHEAP)?"OP_CHEAP":"OP_EXPENSIVE") << "> " << this << std::endl; }
    Operation *copy (void) const { return new GPUOperation (m_opcode); }
};

#endif
