#ifndef __INSTRUCTION_H__
#define __INSTRUCTION_H__

#include <string>
#include <iostream>

/**
 * Simple instruction class.
 */
class Instruction
{
  public:
    /**
     * Create new instruction.
     * @param mnemonic Instruction mnemonic.
     * @param is_arith Whether the instruction is an arithmetic instruction.
     */
    Instruction (const std::string &mnemonic, bool is_arith)
      : m_mnemonic (mnemonic)
      , m_is_arith (is_arith)
    {
    }

  public:
    /**
     * Check whether the instruction is an arithmetic instruction.
     * @return True if so.
     */
    inline bool isArith (void) const { return m_is_arith; }

    /**
     * Check whether the instruction is a memory instruction.
     * @return True if so.
     */
    inline bool isMem (void) const { return !m_is_arith; }

    /**
     * Check whether this instructions conflicts with the given one.
     * @return Always false.
     */
    inline bool conflictsWith (const Instruction *) const { return false; }

  public:
    /**
     * Display instruction.
     */
    void print (void) const { std::cout << m_mnemonic << std::endl; }

  private:
    /// instruction mnemonic
    std::string m_mnemonic;

    /// is arithmetic flag
    bool m_is_arith;
};

#endif
