#include "instruction.h"
#include "reschedule.h"

#include <list>
#include <string>
#include <iostream>

#include <cstdlib>

/**
 * Print instruction sequence.
 * @param title Some title.
 * @param list Instruction sequence.
 */
void print (const std::string &title, std::list<Instruction *> &sequence)
{
  std::cout << title << std::endl;
  for (std::list<Instruction *>::iterator i = sequence.begin(); i != sequence.end(); ++i)
    (*i)->print();
  std::cout << std::endl;
}

/**
 * Reschedule main routine.
 */
int main (int, char **)
{
  // init and display
  std::list<Instruction *> sequence;
  sequence.push_back (new Instruction ("add d1,d2,d3", true));
  sequence.push_back (new Instruction ("add d4,d5,d6", true));
  sequence.push_back (new Instruction ("ld a3,[a2]0", false));
  sequence.push_back (new Instruction ("ld a4,[a5]0", false));
  sequence.push_back (new Instruction ("add d7,d8,d9", true));
  print ("INPUT:", sequence);

  // process
  try
  {
    reschedule::PatternMatcher matcher ("TriCore");
    matcher.setFirstMatch (true);
    matcher.setLocalMinimum (true);
    matcher.process (sequence);
    print ("OUTPUT:", sequence);
  }

  // something bad has happened
  catch (oors::Exception exception)
  {
    std::cerr << exception.msg() << std::endl;
  }

  // cleanup
  for (std::list<Instruction *>::iterator i = sequence.begin(), end = sequence.end(); i != end; ++i)
    delete *i;

  // done
  return 0;
}
