#include "item.h"
#include "sort.h"

#include <oors_internal.h>

#include <list>
#include <string>
#include <iostream>

#include <cstdlib>

#ifdef WIN32
#  include <windows.h>
#else
#  include <sys/time.h>
#endif

/**
 * Debugger for the sorting pattern matcher.
 * Prints the number of swaps after sorting the list.
 */
class Debugger : public oors::Debugger<Item, Item>
{
  public:
    void currentRule (const oors::Rule<Item, Item> *)                                                  {}
    void finishRule (const oors::Rule<Item, Item> *, bool)                                             {}
    void applyRule (const oors::Rule<Item, Item> *)                                                    {}
    void newAlternative (const oors::Alternative<Item, Item> *, const oors::Alternative<Item, Item> *) {}
    void currentAlternative (const oors::Alternative<Item, Item> *)                                    {}
    void deleteAlternative (const oors::Alternative<Item, Item> *)                                     {}
    void matchItemPattern (const oors::Alternative<Item, Item> *, unsigned int, const Item *)          {}
    void matchWildcardPattern (const oors::Alternative<Item, Item> *, unsigned int, const Item *)      {}
    void finishWildcardPattern (const oors::Alternative<Item, Item> *, unsigned int)                   {}
    void finishBasicBlock (int cost)                                                                   { std::cout << "Finished after " << cost << " swaps." << std::endl; }
};

/**
 * Print item list.
 * @param list Item list.
 */
void print (std::list<Item *> &list)
{
  std::cout << list.size() << ": ";
  for (std::list<Item *>::iterator i = list.begin(); i != list.end();)
  {
    std::cout << (*i)->value();
    if (++i != list.end())
      std::cout << ", ";
  }
  std::cout << std::endl;
}

/**
 * Sort main routine.
 * @param argc Argument count.
 * @param argv Argument list.
 */
int main (int argc, char **argv)
{
  // not enough arguments
  if (argc < 3)
  {
    std::cout << "Usage: " << argv[0] << " <Amount> <Ascending|Descending>" << std::endl;
    return 1;
  }

  // init random number generator
#ifdef WIN32
  srand (timeGetTime());
#else
  struct timeval tv;
  gettimeofday (&tv, 0);
  srand ((1000*tv.tv_sec)+(tv.tv_usec/1000));
#endif

  // init and display
  std::list<Item *> list;
  for (int i = 0, size = atoi(argv[1]); i < size; ++i)
    list.push_back (new Item (rand()%100));
  print (list);

  // process
  try
  {
    Debugger debugger;
    sort::PatternMatcher matcher (argv[2]);
    matcher.setFirstMatch (true);
    matcher.setLocalMinimum (true);
    matcher.setDebugger (&debugger);
    matcher.process (list);
    print (list);
  }

  // something bad has happened
  catch (oors::Exception exception)
  {
    std::cerr << exception.msg() << std::endl;
  }

  // cleanup
  for (std::list<Item *>::iterator i = list.begin(), end = list.end(); i != end; ++i)
    delete *i;

  // done
  return 0;
}
